### Bash hooks in livecd-slice.d:

### 300-localdev

localdev_slice()
{
	enter "localdev_slice"

	local odev mp devname="" mnt=""

	if [ -n "$overlays" ]; then
		overlays="${overlays//\\040/ }"
		if [ "$overlays" = local_profile ]; then
			devname="$(stage2_getenv DEVICE)"
			[ -z "$devname" ] ||
				devname="/dev/$devname"
			overlays=
		elif [ -L "/dev/disk/by-label/$overlays" ]; then
			devname="$(readlink-e "/dev/disk/by-label/$overlays" ||:)"
		fi
	fi

	odev="${devname}${overlays:+ (\"$overlays\")}"

	if [ -z "$overlays" ] || [ -z "$devname" ]; then
		debug "overlay device not defined"
		leave "localdev_slice"
		return 0
	elif [ ! -b "$devname" ]; then
		message "specified overlay device not found: $odev"
		leave "localdev_slice"
		return 0
	fi

	message "mounting local overlays for profile '$profile'"
	mp="$(grep -s "$devname /" /proc/mounts |cut -f2 -d' ')"

	if [ -n "$mp" ]; then
		debug "device $odev already mounted to $mp"
	else
		mp="$prevdir.rw/rw$OEM_IMAGES_BASE"

		run mkdir -p -- "$mp"

		if run mount -o ro -- "$devname" "$mp"; then
			debug "device $odev has mounted to $mp"
			mnt=1
		else
			message "can't mount specified overlay device: $odev"
			run rmdir -- "$mp" ||:
			leave "localdev_slice"
			return 0
		fi
	fi

	if [ -d "$mp/$profile" ]; then
		mount_overlays "$mp"
	else
		message "however specified profile not found: '$profile'"

		if [ -n "$mnt" ]; then
			run umount -fl -- "$mp" ||:
			run rmdir -- "$mp" ||:
		fi
	fi

	leave "localdev_slice"
}

case "$method" in
disk|cdrom)
	localdev_slice
	;;
esac

### 700-cifs

cifs_slice()
{
	enter "cifs_slice"

	local server

	server="$(stage2_getenv HOST)"
	server="${server:-$OEM_SRV_NETINST}"

	if [ -z "$server" ]; then
		message "SAMBA server not defined"
		leave "cifs_slice"
		return 0
	fi

	local prefix parentdir="${directory%/?*}"
	local opts mp dirpart share service mnt=

	prefix="$(stage2_getenv PREFIX)"
	opts="$(stage2_getenv SMBOPTS)"
	[ -n "$opts" ] ||
		opts="${CIFSOPTS-}"
	[ -n "$overlays" ] ||
		overlays="${parentdir:-$OEM_CIFS_NETINST}/overlays-live"
	service="${overlays:1}"
	share="${service%%/*}"
	service="//$server/$share"
	dirpart="${overlays#/"$share"}"

	message "mounting CIFS overlays from ${service}${dirpart}"
	mp="$(grep -s "$service /" /proc/mounts |cut -f2 -d' ')"

	if [ -n "$mp" ]; then
		debug "resource $service already mounted to $mp"
	else
		mp="$prevdir.rw/rw$OEM_IMAGES_BASE"

		run mkdir -p -- "$mp"

		if run mount.cifs "$service" "$mp" -o "ro${opts:+,$opts}"; then
			debug "$service has mounted to $mp"
			mnt=1
		else
			message "CIFS mount failed"
			run rmdir -- "$mp" ||:
			leave "cifs_slice"
			return 0
		fi
	fi

	if [ -d "${mp}${dirpart}/$profile" ]; then
		mount_overlays "${mp}${dirpart}"
	else
		message "however specified profile not found: '$profile'"

		if [ -n "$mnt" ]; then
			run umount -fl -- "$mp" ||:
			run rmdir -- "$mp" ||:
		fi
	fi

	leave "cifs_slice"
}

[ "$method" != cifs ] || cifs_slice

### 700-nfs

nfs_slice()
{
	enter "nfs_slice"

	local server dirp

	server="$(stage2_getenv HOST)"
	dirp="$(stage2_getenv PREFIX)"
	[ -n "$dirp" ] || dirp="$directory"

	local parentdir="${dirp%/?*}" mnt=
	local opts="${NFSOPTS:-intr,soft,nolock,timeo=30,retry=0}"
	local mp="$prevdir.rw/rw$OEM_IMAGES_BASE"

	if [ -z "$server" ]; then
		# Fallback: if HOST not exported to the stage2
		message "detecting IP-address of the connected NFS server"
		server="$(run ss -n -o state established '( dport = :2049 )' |
				sed '2!d;s/  */ /g' |
				cut -f5 -d' ' |
				cut -f1 -d:)"
		server="${server:-$OEM_SRV_NETINST}"

		if [ -z "$server" ]; then
			message "NFS server not found"
			leave "nfs_slice"
			return 0
		fi
	fi

	[ -n "$overlays" ] ||
		overlays="${parentdir:-$OEM_NFS_NETINST}/overlays-live"
	message "mounting NFS overlays from $server:$overlays"

	run mkdir -p -- "$mp"

	if run mount.nfs "$server:$overlays" "$mp" -o "ro,$opts"; then
		debug "$server:$overlays has mounted to $mp"
		mnt=1
	else
		message "NFS mount failed"
		run rmdir -- "$mp" ||:
		leave "nfs_slice"
		return 0
	fi

	if [ -d "$mp/$profile" ]; then
		mount_overlays "$mp"
	else
		message "however specified profile not found: '$profile'"

		if [ -n "$mnt" ]; then
			run umount -fl -- "$mp" ||:
			run rmdir -- "$mp" ||:
		fi
	fi

	leave "nfs_slice"
}

[ "$method" != nfs ] || nfs_slice

