### Bash hooks in translate.d:

### 050-overlays

if [ -n "$overlays" ] || [ -n "${PROFILE-}" ]; then
	overlays="overlays=$overlays;profile=${PROFILE-}"
	[ -z "$directory" ] ||
		overlays="$overlays;directory=$directory"
fi

### 100-cifs

if [ "$method" = cifs ]; then
	VALUE="server=$server"
	[ -z "$directory" ] ||
		VALUE="$VALUE;directory=$directory"
	[ -z "$timeout" ] ||
		VALUE="$VALUE;timeout=$timeout"
	[ -z "$options" ] ||
		VALUE="$VALUE;options=$options"
	[ -z "$domain" ] ||
		VALUE="$VALUE;domain=$domain"
	[ -z "$user" ] ||
		VALUE="$VALUE;user=$user;pass=$pass"
	ALTBOOT_CIFS0="$VALUE"
	ALTBOOT="waitnet,cifs"
	VALUE=
fi

### 100-download

case "$method" in
ftp|http|url)
	ALTBOOT="waitnet,download"
	if grep -qws ramdisk_size /proc/cmdline; then
		VALUE="method=$method;to=RD;fuid=$STAGENAME"
	elif [ "$type" != iso ]; then
		VALUE="method=$method;to=TMPFS;fuid=$STAGENAME"
	else
		if [ -n "${HASH-}" ]; then
			ALTBOOT="$ALTBOOT,checksum"
			ALTBOOT_CHECKSUM0="$HASH"
			HASH=
		fi
		VALUE="method=$method;to=TMPFS"
		ALTBOOT="$ALTBOOT,iso9660"
	fi
	if [ "$method" = url ]; then
		VALUE="$VALUE;url=$url"
	elif [ -n "$server" ] || [ -n "$directory" ]; then
		VALUE="$VALUE;server=$server;directory=$directory"
		[ -z "$user" ] || [ "$method" = http ] ||
			VALUE="$VALUE;user=$user;pass=$pass"
	fi
	[ -z "$imgsize" ] ||
		VALUE="$VALUE;imgsize=$imgsize"
	[ -z "$timeout" ] ||
		VALUE="$VALUE;timeout=$timeout"
	ALTBOOT_DOWNLOAD0="$VALUE"
	overlays=
	LOWMEM=1
	VALUE=
	;;
esac

### 100-localdev

case "$method" in
cdrom|disk)
	ALTBOOT=localdev
	VALUE="method=$method"
	if [ "$method" = cdrom ]; then
		directory="/$STAGENAME"
		partition=
	fi
	[ -z "$disk" ] ||
		VALUE="$VALUE;disk=$disk"
	[ -z "$partition" ] ||
		VALUE="$VALUE;part=$partition"
	[ -z "$fuid" ] ||
		VALUE="$VALUE;fuid=$fuid"
	[ -z "$uuid" ] ||
		VALUE="$VALUE;uuid=$uuid"
	[ -z "$label" ] ||
		VALUE="$VALUE;label=$label"
	[ -z "$directory" ] ||
		VALUE="$VALUE;directory=$directory"
	[ -z "$options" ] ||
		VALUE="$VALUE;options=$options"
	[ -z "$timeout" ] ||
		VALUE="$VALUE;timeout=$timeout"
	[ -z "${UPDATEMODULES-}" ] ||
		ALTBOOT="oemsetup,$ALTBOOT"
	ALTBOOT_LOCALDEV0="$VALUE"
	VALUE=
	;;
esac

### 100-nfs

if [ "$method" = nfs ]; then
	VALUE="server=$server"
	[ -z "$directory" ] ||
		VALUE="$VALUE;directory=$directory"
	[ -z "$timeout" ] ||
		VALUE="$VALUE;timeout=$timeout"
	[ -z "$options" ] ||
		VALUE="$VALUE;options=$options"
	ALTBOOT_NFS0="$VALUE"
	ALTBOOT="waitnet,nfs"
	VALUE=
fi

### 350-squashfs

if [ -n "${LOWMEM-}" ]; then
	if [ -n "${HASH-}" ]; then
		ALTBOOT="$ALTBOOT,checksum"
		ALTBOOT_CHECKSUM0=":$HASH:/$STAGENAME"
	fi

	ALTBOOT="$ALTBOOT,squashfs"
	ALTBOOT_SQUASHFS0="/$STAGENAME"
else
	ALTBOOT="$ALTBOOT,copyfile"
	ALTBOOT_COPYFILE0="src=/$STAGENAME"
	grep -qws ramdisk_size /proc/cmdline ||
		ALTBOOT_COPYFILE0="dst=TMPFS;$ALTBOOT_COPYFILE0"

	if [ -n "${HASH-}" ]; then
		ALTBOOT="$ALTBOOT,checksum"
		ALTBOOT_CHECKSUM0="$HASH"
	fi

	ALTBOOT="$ALTBOOT,squashfs"
	ALTBOOT_SQUASHFS0="-"
fi

### 520-liverw

[ "$STAGENAME" = rescue ] || [ "$STAGENAME" = live ] || LIVE_RW=

### 550-liveboot

if [ "$STAGENAME" != rescue ] && [ "$STAGENAME" != live ]; then
	RESCUE=
	LIVE=
fi

VALUE=
ALTBOOT_LIVEBOOT0="stagename=$STAGENAME;method=$method"
[ "$STAGENAME" != live ] && [ -z "${LIVE-}" ] ||
	VALUE=live_ro
[ "$STAGENAME" != rescue ] && [ -z "${RESCUE-}" ] ||
	VALUE="${VALUE:+$VALUE,}rescue"
[ -z "$overlays" ] ||
	ALTBOOT_LIVEBOOT0="$ALTBOOT_LIVEBOOT0;$overlays"

### 650-liverw

[ -z "${LIVE_RW-}" ] ||
	VALUE="${VALUE:+$VALUE,}live_rw"

### 750-liveboot

[ -z "$VALUE" ] ||
	ALTBOOT_LIVEBOOT0="$ALTBOOT_LIVEBOOT0;flags=$VALUE"
[ -z "$timeout" ] ||
	ALTBOOT_LIVEBOOT0="$ALTBOOT_LIVEBOOT0;timeout=$timeout"

### 950-liveboot

ALTBOOT="$ALTBOOT,liveboot"

